/*

For use with SQLLab XTP analysis

NOTE: Added coverage of AUTO_UPDATE_STATISTICS database option 
included below.

SQL Server 2016 has Auto Update Statistics on by default for in-memory tables.

Create 3 sets of tables:
1. In-Memory with HASH indexes
2. In-Memory with NONCLUSTERED indexes
3. Disk-based with NONCLUSTERED indexes

*/



USE SQLLab

IF OBJECT_ID('maintable_regular') IS NOT NULL 
	DROP TABLE SQLLab.dbo.maintable_regular

IF OBJECT_ID('lookup3table_regular') IS NOT NULL
	DROP TABLE SQLLab.dbo.lookup3table_regular

IF OBJECT_ID('lookup2table_regular') IS NOT NULL
	DROP TABLE SQLLab.dbo.lookup2table_regular

IF OBJECT_ID('lookup1table_regular') IS NOT NULL
	DROP TABLE SQLLab.dbo.lookup1table_regular

IF OBJECT_ID('maintable_inmemory') IS NOT NULL
	DROP TABLE SQLLab.dbo.maintable_inmemory

IF OBJECT_ID('lookup1table_inmemory') IS NOT NULL
	DROP TABLE SQLLab.dbo.lookup1table_inmemory

IF OBJECT_ID('lookup2table_inmemory') IS NOT NULL
	DROP TABLE SQLLab.dbo.lookup2table_inmemory

IF OBJECT_ID('lookup3table_inmemory') IS NOT NULL
	DROP TABLE SQLLab.dbo.lookup3table_inmemory

IF OBJECT_ID('maintable_inmemoryNC') IS NOT NULL
	DROP TABLE maintable_inmemoryNC

IF OBJECT_ID('lookup1table_inmemoryNC') IS NOT NULL
	DROP TABLE lookup1table_inmemoryNC

IF OBJECT_ID('lookup2table_inmemoryNC') IS NOT NULL
	DROP TABLE lookup2table_inmemoryNC

IF OBJECT_ID('lookup3table_inmemoryNC') IS NOT NULL
	DROP TABLE lookup3table_inmemoryNC 

-- Main table for disk-based storage
CREATE TABLE maintable_regular (
						[col1] [int] NOT NULL primary key nonclustered,
                        [col2] [int] NOT NULL,
                        [col3] [int] NULL,
                        [col4] [varchar](50) NULL,
						code1	varchar(50) NULL,
						code2	varchar(50) NULL,
						code3	varchar(50) null
						INDEX hlu1 NONCLUSTERED (col2,code1),
						INDEX hlu2 NONCLUSTERED (col2,code2),
						INDEX hlu3 NONCLUSTERED (col2,code3) 
						)
						; 


-- Lookup table 1 for disk-based storage
CREATE TABLE lookup1table_regular (
						[col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
                        [code1]	varchar(50) NOT NULL
						INDEX lu1 NONCLUSTERED (col2,code1)
						)
						;

-- Lookup table 2 for disk-based storage
CREATE TABLE lookup2table_regular (
						[col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
						[code2]	varchar(50) NOT NULL
						INDEX lu2 NONCLUSTERED (col2,code2)
						)
						;
-- Lookup table 3 for disk-based storage
CREATE TABLE lookup3table_regular (
						[col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
						[code3]	varchar(50) NOT NULL
						INDEX lu3 NONCLUSTERED (col2,code3)
						)
						;


-- Table creation for in-memory storage tables with HASH index
CREATE TABLE maintable_inmemory (
						[col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
                        [col3] [int] NULL,
                        [col4] [varchar](50) NULL,
						code1	varchar(50) NULL,
						code2	varchar(50) NULL,
						code3	varchar(50) null

						INDEX hlu1 NONCLUSTERED HASH (col2,code1) WITH (BUCKET_COUNT = 32),
						INDEX hlu2 NONCLUSTERED HASH (col2,code2) WITH (BUCKET_COUNT = 32),
						INDEX hlu3 NONCLUSTERED HASH (col2,code3) WITH (BUCKET_COUNT = 32)
						) WITH (MEMORY_OPTIMIZED=ON)
						;
CREATE TABLE lookup1table_inmemory ([col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
                        [code1]	varchar(50) NULL
						
						,INDEX ilu1 NONCLUSTERED HASH(col2, code1) WITH (BUCKET_COUNT =16)
						) WITH (MEMORY_OPTIMIZED=ON)
						;
CREATE TABLE lookup2table_inmemory ([col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
						[code2]	varchar(50) NULL

						,INDEX ilu2 nonclustered HASH(col2, code2) WITH (BUCKET_COUNT =16)						
						) WITH (MEMORY_OPTIMIZED=ON)
						;
CREATE TABLE lookup3table_inmemory ([col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
						[code3] varchar(50) NULL
						
						,INDEX ilu3 nonclustered HASH(col2, code3) WITH (BUCKET_COUNT =16)						
						) WITH (MEMORY_OPTIMIZED=ON)
						;

-- Table creation for in-memory storage tables with NON-CLUSTERED index 
CREATE TABLE maintable_inmemoryNC ([col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
                        [col3] [int] NULL,
                        [col4] [varchar](50) NULL,
						code1	varchar(50) NULL,
						code2	varchar(50) NULL,
						code3	varchar(50) null

						INDEX hlu1 NONCLUSTERED (col2,code1) ,
						INDEX hlu2 NONCLUSTERED (col2,code2) ,
						INDEX hlu3 NONCLUSTERED (col2,code3) 
						) WITH (MEMORY_OPTIMIZED=ON)
						;

CREATE TABLE lookup1table_inmemoryNC ([col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
                        [code1]	varchar(50) NULL
						
						,INDEX ilu1 NONCLUSTERED  (col2, code1)  
						) WITH (MEMORY_OPTIMIZED=ON)
						;

CREATE TABLE lookup2table_inmemoryNC ([col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
						[code2]	varchar(50) NULL

						,INDEX ilu2 nonclustered  (col2, code2)  					
						) WITH (MEMORY_OPTIMIZED=ON)
						;

CREATE TABLE lookup3table_inmemoryNC ([col1] [int] NOT NULL primary key nonclustered,
						[col2] int NOT NULL,
						[code3] varchar(50) NULL
						
						,INDEX ilu3 nonclustered  (col2, code3)  					
						) WITH (MEMORY_OPTIMIZED=ON)
						;
