/*

For use with SQLLab XTP analysis

NOTE: Added coverage of AUTO_UPDATE_STATISTICS database option 
included below.

SQL Server 2016 has Auto Update Statistics on by default for in-memory tables.

Load data into  3 sets of tables:
1. In-Memory with HASH indexes
2. In-Memory with NONCLUSTERED indexes
3. Disk-based with NONCLUSTERED indexes

*/

USE SQLLab 

DECLARE @maxcount INT = 200000	-- each of 3 main tables will have 200,000 rows each.
DECLARE @val INT
SELECT @val=1

delete from maintable_regular;

WHILE @val <= @maxcount
BEGIN  
   INSERT INTO maintable_regular (col1, col2, col3, col4, code1, code2, code3) 
       VALUES	(@val, (@val+1) % 12, @val,
					CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97) + CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97) + 
					CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97) + CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97) + 
					CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97) + CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97) + 
					CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97) + CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97) + 
					CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97) + CHAR((ABS(CHECKSUM(NEWID())) % 26) + 97),
					'code1-' + CAST( (@val + 1) % 2 AS VARCHAR),
					'code2-' + CAST( (@val + 1) % 3 AS VARCHAR),
					'code3-' + CAST( (@val + 1) % 12 AS VARCHAR)
				)
				;

						
   SELECT @val=@val+1
END


	DELETE  FROM lookup1table_regular;
	DELETE  FROM lookup2table_regular;
	DELETE  FROM lookup3table_regular;

	INSERT INTO lookup1table_regular(col1, col2, code1)
		VALUES	(1, 0, 'code1-0'), 
				(2, 1, 'code1-1')
				;
	INSERT INTO lookup2table_regular(col1, col2, code2)
		VALUES	(1, 0, 'code2-0'), 
				(2, 1, 'code2-1'), 
				(3, 2, 'code2-2') 
				;
	INSERT INTO lookup3table_regular(col1, col2, code3)
		VALUES	(1, 0, 'code3-0'), 
				(2, 1, 'code3-1'), 
				(3, 2, 'code3-2') , 
				(4, 3, 'code3-3')
				;	

--- Load in-memory HASH indexed tables with sample data
delete from maintable_inmemory;

insert into maintable_inmemory		--- load same 200,000 rows into in-memory HASH main table
SELECT * FROM maintable_regular;

	DELETE  FROM lookup1table_inmemory;
	DELETE  FROM lookup2table_inmemory;
	DELETE  FROM lookup3table_inmemory;

	INSERT INTO lookup1table_inmemory
		SELECT * FROM lookup1table_regular
				;
	INSERT INTO lookup2table_inmemory
		SELECT * FROM lookup2table_regular
				;
	INSERT INTO lookup3table_inmemory 
		SELECT * FROM lookup3table_regular


--- Load in-memory NON-CLUSTERED indexed tables with sample data
delete from maintable_inmemoryNC;

insert into maintable_inmemoryNC	--- load same 200,000 rows into in-memory NON-CLUSTERED main table
SELECT * FROM maintable_regular;


	DELETE  FROM lookup1table_inmemoryNC;
	DELETE  FROM lookup2table_inmemoryNC;
	DELETE  FROM lookup3table_inmemoryNC;

	INSERT INTO lookup1table_inmemoryNC
		SELECT * FROM lookup1table_regular
				;
	INSERT INTO lookup2table_inmemoryNC
		SELECT * FROM lookup2table_regular
				;
	INSERT INTO lookup3table_inmemoryNC 
		SELECT * FROM lookup3table_regular


RETURN;




/*
Starting in SQL Server 2016 and in Azure SQL Database, automatic update of statistics 
is supported for memory-optimized tables, when using database compatibility level of at least 130. 

Statistics are created automatically, but can also be created manually.

*/
UPDATE STATISTICS [dbo].[maintable_inmemory];
UPDATE STATISTICS [dbo].[lookup1table_inmemory];
UPDATE STATISTICS [dbo].[lookup2table_inmemory];
UPDATE STATISTICS [dbo].[lookup3table_inmemory];

-- Assuming AUTO_UPDATE_STATISTICS is already ON for your database:
-- ALTER DATABASE CURRENT SET AUTO_UPDATE_STATISTICS ON;

ALTER DATABASE CURRENT SET COMPATIBILITY_LEVEL = 130;
GO
DECLARE @sql NVARCHAR(MAX) = N'';
SELECT
      @sql += N'UPDATE STATISTICS '
         + quotename(schema_name(t.schema_id))
         + N'.'
         + quotename(t.name)
         + ';' + CHAR(13) + CHAR(10)
   FROM sys.tables AS t
   WHERE t.is_memory_optimized = 1 AND 
        t.object_id IN (SELECT object_id FROM sys.stats WHERE no_recompute=1)
;
EXECUTE sp_executesql @sql;
GO
-- Each row appended to @sql looks roughly like:
-- UPDATE STATISTICS [dbo].[MyMemoryOptimizedTable];




/*
Verify auto-update is enabled: The following script verifies whether automatic update is enabled for statistics 
on memory-optimized tables. After running the previous script it will return 1 in the column auto-update 
enabled for all statistic objects.
*/

SELECT 
    quotename(schema_name(o.schema_id)) + N'.' + quotename(o.name) AS [table],
    s.name AS [statistics object],
    1-s.no_recompute AS [auto-update enabled]
FROM sys.stats s JOIN sys.tables o ON s.object_id=o.object_id
WHERE o.is_memory_optimized=1